"""
A demo of using chemkin.py to calculate the reaction rates of a system of 
irreversible elementary reactions by reading reaction data stored in XML
files.

"""

from chemkin.reaction.reaction_coefficients import *
from chemkin.reaction.elementary_rxn import *
from chemkin.thermodynamics.thermo import  *
from chemkin.preprocessing.parse_xml import XmlParser

from chemkin.thermodynamics.thermo import *

#import preprocessing.tests.test_parse_xml

xml_file = './xml-files/rxns_rev.xml'
xml_parser = XmlParser(xml_file)


species, rxn_data_list = xml_parser.load()
n_species = len(species)

xi = [2.0, 1.0, 0.5, 1.0, 1.0] # specii concentration

species_idx_dict = {} # build the dictionary of key = species_name, value = species_index
for i, s in enumerate(species):
	species_idx_dict[s] = i


Ti = [750, 1500, 2500]

for T in Ti:
	sys_vi_p = [] # list of reactant Stoichiometric coefficients in each rxn
	sys_vi_dp = [] # list of product Stoichiometric coefficients in each rxn
	ki = [] # list of reation rate coefficients in each rxn
	is_reversible = None # indicator of the system of reactions being irreversible/reversible

	for rxn_data in rxn_data_list: # 1 rxn per rxn_data
		if rxn_data.type != RxnType.Elementary: 
			raise TypeError('Non-elementary reactions not implemented yet.')

		if is_reversible == None:
			is_reversible = rxn_data.reversible # set the indicator of the system of reactions to be irreversible/reversible

		if rxn_data.reversible != is_reversible: # the system of reactions in the XML file must be all irreversible/reversible
			raise TypeError('The system of reactions are inconsistent in reversibility.')
		
		rxn_id = rxn_data.rxn_id # save id

		rxn_vi_p = np.zeros((n_species,)) # save the Stoichiometric coefficients of the reactants in this rxn
		for s, vi in rxn_data.reactants.items():
			idx = species_idx_dict[s] # get index of the specii
			rxn_vi_p[idx] = vi
		sys_vi_p.append(list(rxn_vi_p))

		rxn_vi_dp = np.zeros((n_species,)) # save the Stoichiometric coefficients of the products in this rxn
		for s, vi in rxn_data.products.items():
			idx = species_idx_dict[s] # get index of the specii
			rxn_vi_dp[idx] = vi
		sys_vi_dp.append(list(rxn_vi_dp))
		
		coef_params = rxn_data.rate_coeff
		if isinstance(coef_params, list):
			if len(coef_params) == 3: # modified arrhenius coef
				A = coef_params[0]
				b = coef_params[1]
				E = coef_params[2]
				ki.append(ModArrheniusCoef(A, b, E, T).get_coef())
			else: # arrhenius coef
				A = coef_params[0]
				E = coef_params[1]
				ki.append(ArrheniusCoef(A, E, T).get_coef())
		else: # const coef
			ki.append(ConstCoef(coef_params).get_coef())

	# print(sys_vi_p)
	# print(sys_vi_dp)
	# print(ki)
	# print(IrrevElemRxn(ki, xi, sys_vi_p, sys_vi_dp))
	if is_reversible == False:
		rxn_rates = IrrevElemRxn(ki, xi, sys_vi_p, sys_vi_dp).reaction_rate()
	else:
		b_ki = Thermo(species, T, ki, sys_vi_p, sys_vi_dp).get_backward_coefs()
		rxn_rates = RevElemRxn(ki, b_ki, xi, sys_vi_p, sys_vi_dp).reaction_rate()

	

	print('------At Temperature', T, 'K------')
	for s, rate in zip(species, rxn_rates):
		print('    ', s, ':', rate)
	print('--------------------------------')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 13 21:33:30 2017

@author: filipmichalsky
"""

import sqlite3
import numpy as np

class Thermo():
    """ Class of thermodynanmics
    """

    def __init__ (self, species, T, ki, vi_p, vi_dp, db_name='NASA_coef.sqlite'):
        self.species = species
        self.T = T
        self.ki = ki
        self.b_ki = None
        self.vi_p = vi_p
        self.vi_dp = vi_dp
        self.dao = ThermoDAO(db_name)
        
    def get_backward_coefs (self):
        vi_p = np.matrix(self.vi_p).T
        vi_dp = np.matrix(self.vi_dp).T
        vi = vi_dp - vi_p  # calculate overall stoicheometric coefficients

        H_species = []
        S_species = []
        for s in self.species:
            if self.T >= 1000: # high temperature range
                NASA_coefs = self.dao.get_coeffs(s, 'high')
            else: # low temperature range
                NASA_coefs = self.dao.get_coeffs(s, 'low')

            # Compute enthalpy/(RT) of a specie
            H_T_arr = np.array([1, 1/2*self.T, 1/3*(self.T**2), 1/4*(self.T**3), 1/5*(self.T**4), 1/self.T])
            H_coef_arr = np.array([NASA_coefs[0], NASA_coefs[1], NASA_coefs[2], NASA_coefs[3], NASA_coefs[4], NASA_coefs[5]])
            H_species.append(np.dot(H_T_arr, H_coef_arr))

            # Compute entropy/R of a specie
            S_T_arr = np.array([np.log(self.T), self.T, 1/2*(self.T**2), 1/3*(self.T**3), 1/4*(self.T**4), 1])
            S_coef_arr = np.array([NASA_coefs[0], NASA_coefs[1], NASA_coefs[2], NASA_coefs[3], NASA_coefs[4], NASA_coefs[6]])
            S_species.append(np.dot(S_T_arr, S_coef_arr))

        delta_H = np.squeeze(np.asarray(np.dot(H_species, vi))) # delta enthalpy of a reaction
        delta_S = np.squeeze(np.asarray(np.dot(S_species, vi))) # delta entropy of a reaction

        gamma = np.sum(vi, axis=0)
        
        p0 = 10e5
        R = 8.314
        ke = np.squeeze(np.asarray(np.multiply(np.power(p0/(R*self.T), gamma), np.exp(delta_S-delta_H))))

        self.b_ki = self.ki/ke

        return self.b_ki


class ThermoDAO():
    """ Database Access Object for thermodynanmics
    """

    def __init__ (self, db_name='NASA_coef.sqlite'):
        self.db = sqlite3.connect(db_name)
        self.cursor = self.db.cursor()


    def get_coeffs (self, species_name, temp_range):
        query = '''SELECT COEFF_1, COEFF_2, COEFF_3, COEFF_4, COEFF_5, COEFF_6, COEFF_7
                    FROM {} 
                    WHERE SPECIES_NAME = "{}"'''.format(temp_range.upper(), species_name)
        coeffs = list(self.cursor.execute(query).fetchall()[0])
        return coeffs



from setuptools import setup

setup(name='chemkin',
      version='1.3',
      description='The best chemkin package',
      url='https://github.com/G12-cs207-FinalProject/cs207-FinalProject.git',
      author='Parser',
      license='Harvard',
      packages=['chemkin',
                'chemkin.preprocessing',
                'chemkin.preprocessing.tests',
                'chemkin.reaction',
                'chemkin.reaction.tests',
                'chemkin.thermodynamics'],
    package_data={'chemkin': ['*.sqlite',
                                'xml-files/*.xml']})